@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """The main interface to the allegedb ORM, and some supporting functions and classes"""
-from contextlib import ContextDecorator
+from contextlib import ContextDecorator, contextmanager
 from weakref import WeakValueDictionary
 
 from blinker import Signal
@@ -26,7 +26,8 @@ from .graph import (
     MultiGraph,
     MultiDiGraph,
     Node,
-    Edge
+    Edge,
+    GraphsMapping
 )
 from .query import QueryEngine, TimeError
 from .window import HistoryError
@@ -52,72 +53,72 @@ class PlanningContext(ContextDecorator):
     of the plan, in which case the world will obey those, and cancel any
     future plan.
 
-    New branches cannot be started within plans.
+    New branches cannot be started within plans. The ``with orm.forward():``
+    optimization is disabled within a ``with orm.plan():`` block, so
+    consider another approach instead of making a very large plan.
 
     """
-    __slots__ = ['orm', 'time']
+    __slots__ = ['orm', 'id', 'forward']
 
     def __init__(self, orm):
         self.orm = orm
 
     def __enter__(self):
-        if self.orm._planning:
+        orm = self.orm
+        if orm._planning:
             raise ValueError("Already planning")
-        self.orm._planning = True
-        self.time = self.orm.btt()
+        orm._planning = True
+        branch, turn, tick = orm._btt()
+        self.id = myid = orm._last_plan = orm._last_plan + 1
+        self.forward = orm._forward
+        if orm._forward:
+            orm._forward = False
+        orm._plans[myid] = branch, turn, tick
+        orm._plans_uncommitted.append((myid, branch, turn, tick))
+        orm._branches_plans[branch].add(myid)
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.orm._obranch, self.orm._oturn, self.orm._otick = self.time
         self.orm._planning = False
+        if self.forward:
+            self.orm._forward = True
 
 
-class TimeSignal(Signal):
-    """Acts like a list of ``[branch, turn]`` for the most part.
+class TimeSignal:
+    """Acts like a tuple of ``(branch, turn)`` for the most part.
 
-    You can set these to new values, or even replace them with a whole new
-    ``[branch, turn]`` if you wish. It's even possible to use the strings
-    ``'branch'`` or ``'turn'`` in the place of indices, but at that point
-    you might prefer to set ``engine.branch`` or ``engine.turn`` directly.
-
-    This is a Signal, so pass a function to the connect(...) method and
-    it will be called whenever the time changes. Not when the tick
-    changes, though. If you really need something done whenever the
-    tick changes, override the _set_tick method of
-    :class:`allegedb.ORM`.
+    This wraps a ``Signal``. To set a function to be called whenever the branch
+    or turn changes, pass it to my ``connect`` method.
 
     """
-    def __init__(self, engine):
-        super().__init__()
+    def __init__(self, engine, sig):
         self.engine = engine
+        self.branch = self.engine.branch
+        self.turn = self.engine.turn
+        self.sig = sig
 
     def __iter__(self):
-        yield self.engine.branch
-        yield self.engine.turn
+        yield self.branch
+        yield self.turn
 
     def __len__(self):
         return 2
 
     def __getitem__(self, i):
         if i in ('branch', 0):
-            return self.engine.branch
+            return self.branch
         if i in ('turn', 1):
-            return self.engine.turn
+            return self.turn
+        raise IndexError
 
-    def __setitem__(self, i, v):
-        branch_then, turn_then, tick_then = self.engine.btt()
-        if i in ('branch', 0):
-            self.engine.branch = v
-        if i in ('turn', 1):
-            self.engine.turn = v
-        branch_now, turn_now, tick_now = self.engine.btt()
-        self.send(
-            self, branch_then=branch_then, turn_then=turn_then, tick_then=tick_then,
-            branch_now=branch_now, turn_now=turn_now, tick_now=tick_now
-        )
+    def connect(self, *args, **kwargs):
+        self.sig.connect(*args, **kwargs)
+
+    def send(self, *args, **kwargs):
+        self.sig.send(*args, **kwargs)
 
     def __str__(self):
-        return str((self.engine.branch, self.engine.turn))
+        return str(tuple(self))
 
     def __eq__(self, other):
         return tuple(self) == other
@@ -144,20 +145,20 @@ class TimeSignalDescriptor:
 
     def __get__(self, inst, cls):
         if id(inst) not in self.signals:
-            self.signals[id(inst)] = TimeSignal(inst)
-        return self.signals[id(inst)]
+            self.signals[id(inst)] = Signal()
+        return TimeSignal(inst, self.signals[id(inst)])
 
     def __set__(self, inst, val):
         if id(inst) not in self.signals:
-            self.signals[id(inst)] = TimeSignal(inst)
-        real = self.signals[id(inst)]
-        branch_then, turn_then, tick_then = real.engine.btt()
+            self.signals[id(inst)] = Signal()
+        sig = self.signals[id(inst)]
+        branch_then, turn_then, tick_then = inst._btt()
         branch_now, turn_now = val
         if (branch_then, turn_then) == (branch_now, turn_now):
             return
-        e = real.engine
+        e = inst
         # enforce the arrow of time, if it's in effect
-        if e._forward:
+        if e._forward and not e._planning:
             if branch_now != branch_then:
                 raise TimeError("Can't change branches in a forward context")
             if turn_now < turn_then:
@@ -207,7 +208,7 @@ class TimeSignalDescriptor:
             if tick_now > e._turn_end[val]:
                 e._turn_end[val] = tick_now
         e._otick = e._turn_end_plan[val] = tick_now
-        real.send(
+        sig.send(
             e,
             branch_then=branch_then,
             turn_then=turn_then,
@@ -289,34 +290,44 @@ class ORM(object):
         return self.node_cls(graph, node)
 
     def _get_node(self, graph, node):
-        key = (graph.name, node)
-        if key in self._node_objs:
-            return self._node_objs[key]
-        if not self._node_exists(graph.name, node):
-            self._exist_node(graph.name, node)
-        ret = self._make_node(graph, node)
-        self._node_objs[key] = ret
+        node_objs, node_exists, make_node = self._get_node_stuff
+        if type(graph) is str:
+            graphn = graph
+            graph = self.graph[graphn]
+        else:
+            graphn = graph.name
+        key = (graphn, node)
+        if key in node_objs:
+            return node_objs[key]
+        if not node_exists(graphn, node):
+            raise KeyError("No such node: {} in {}".format(node, graphn))
+        ret = make_node(graph, node)
+        node_objs[key] = ret
         return ret
 
     def _make_edge(self, graph, orig, dest, idx):
         return self.edge_cls(graph, orig, dest, idx)
 
     def _get_edge(self, graph, orig, dest, idx=0):
-        key = (graph.name, orig, dest, idx)
-        if key in self._edge_objs:
-            return self._edge_objs[key]
-        if not self._edge_exists(graph.name, orig, dest, idx):
-            self._exist_edge(graph.name, orig, dest, idx)
-        ret = self._make_edge(graph, orig, dest, idx)
-        self._edge_objs[key] = ret
+        edge_objs, edge_exists, make_edge = self._get_edge_stuff
+        if type(graph) is str:
+            graphn = graph
+            graph = self.graph[graphn]
+        else:
+            graphn = graph.name
+        key = (graphn, orig, dest, idx)
+        if key in edge_objs:
+            return edge_objs[key]
+        if not edge_exists(graphn, orig, dest, idx):
+            raise KeyError("No such edge: {}->{}[{}] in {}".format(orig, dest, idx, graphn))
+        ret = make_edge(graph, orig, dest, idx)
+        edge_objs[key] = ret
         return ret
 
     def plan(self):
         return PlanningContext(self)
     plan.__doc__ = PlanningContext.__doc__
 
-    from contextlib import contextmanager
-
     @contextmanager
     def advancing(self):
         """A context manager for when time is moving forward one turn at a time.
@@ -405,6 +416,10 @@ class ORM(object):
         to node and edge attributes, and 'nodes' and 'edges' full of booleans
         indicating whether a node or edge exists.
 
+        :arg branch: A branch of history; defaults to the current branch
+        :arg turn: The turn in the branch; defaults to the current turn
+        :arg tick_from: Starting tick; defaults to 0
+
         """
         branch = branch or self.branch
         turn = turn or self.turn
@@ -487,9 +502,12 @@ class ORM(object):
     def _init_caches(self):
         from collections import defaultdict
         from .cache import Cache, NodesCache, EdgesCache
+        self._where_cached = defaultdict(list)
         self._global_cache = self.query._global_cache = {}
-        self._node_objs = WeakValueDictionary()
-        self._edge_objs = WeakValueDictionary()
+        self._node_objs = node_objs = WeakValueDictionary()
+        self._get_node_stuff = (node_objs, self._node_exists, self._make_node)
+        self._edge_objs = edge_objs = WeakValueDictionary()
+        self._get_edge_stuff = (edge_objs, self._edge_exists, self._make_edge)
         for k, v in self.query.global_items():
             if k == 'branch':
                 self._obranch = v
@@ -509,12 +527,28 @@ class ORM(object):
         """Tick on which a (branch, turn) ends"""
         self._turn_end_plan = defaultdict(lambda: 0)
         """Tick on which a (branch, turn) ends, even if it hasn't been simulated"""
-        self._graph_val_cache = Cache(self)
-        self._nodes_cache = NodesCache(self)
-        self._edges_cache = EdgesCache(self)
-        self._node_val_cache = Cache(self)
-        self._edge_val_cache = Cache(self)
         self._graph_objs = {}
+        self._plans = {}
+        self._branches_plans = defaultdict(set)
+        self._plan_ticks = defaultdict(lambda: defaultdict(list))
+        self._time_plan = {}
+        self._plans_uncommitted = []
+        self._plan_ticks_uncommitted = []
+        self._graph_val_cache = Cache(self)
+        self._graph_val_cache.setdb = self.query.graph_val_set
+        self._graph_val_cache.deldb = self.query.graph_val_del_time
+        self._nodes_cache = NodesCache(self)
+        self._nodes_cache.setdb = self.query.exist_node
+        self._nodes_cache.deldb = self.query.nodes_del_time
+        self._edges_cache = EdgesCache(self)
+        self._edges_cache.setdb = self.query.exist_edge
+        self._edges_cache.deldb = self.query.edges_del_time
+        self._node_val_cache = Cache(self)
+        self._node_val_cache.setdb = self.query.node_val_set
+        self._node_val_cache.deldb = self.query.node_val_del_time
+        self._edge_val_cache = Cache(self)
+        self._edge_val_cache.setdb = self.query.edge_val_set
+        self._edge_val_cache.deldb = self.query.edge_val_del_time
 
     def _load_graphs(self):
         for (graph, typ) in self.query.graphs_types():
@@ -535,6 +569,14 @@ class ORM(object):
         """Make a SQLAlchemy engine if possible, else a sqlite3 connection. In
         either case, begin a transaction.
 
+        :arg dbstring: rfc1738 URL for a database connection. Unless it begins with
+        "sqlite:///", SQLAlchemy will be required.
+        :arg alchemy: Set to ``False`` to use the precompiled SQLite queries even if
+        SQLAlchemy is available.
+        :arg connect_args: Dictionary of keyword arguments to be used for the database
+        connection.
+        :arg validate: Whether to perform an integrity test on the data.
+
         """
         self._planning = False
         self._forward = False
@@ -546,6 +588,7 @@ class ORM(object):
             )
         self.query.initdb()
         # in case this is the first startup
+        self._obranch = 'trunk'
         self._otick = self._oturn = 0
         self._init_caches()
         for (branch, parent, parent_turn, parent_tick, end_turn, end_tick) in self.query.all_branches():
@@ -568,22 +611,36 @@ class ORM(object):
 
     def _init_load(self, validate=False):
         if not hasattr(self, 'graph'):
-            self.graph = self._graph_objs
+            self.graph = GraphsMapping(self)
         noderows = [
             (graph, node, branch, turn, tick, ex if ex else None)
             for (graph, node, branch, turn, tick, ex)
             in self.query.nodes_dump()
         ]
-        self._nodes_cache.load(noderows, validate=validate)
+        self._nodes_cache.load(noderows)
         edgerows = [
             (graph, orig, dest, idx, branch, turn, tick, ex if ex else None)
             for (graph, orig, dest, idx, branch, turn, tick, ex)
             in self.query.edges_dump()
         ]
-        self._edges_cache.load(edgerows, validate=validate)
-        self._graph_val_cache.load(self.query.graph_val_dump(), validate=validate)
-        self._node_val_cache.load(self.query.node_val_dump(), validate=validate)
-        self._edge_val_cache.load(self.query.edge_val_dump(), validate=validate)
+        self._edges_cache.load(edgerows)
+        self._graph_val_cache.load(self.query.graph_val_dump())
+        self._node_val_cache.load(self.query.node_val_dump())
+        self._edge_val_cache.load(self.query.edge_val_dump())
+        last_plan = -1
+        plans = self._plans
+        branches_plans = self._branches_plans
+        for plan, branch, turn, tick in self.query.plans_dump():
+            plans[plan] = branch, turn, tick
+            branches_plans[branch].add(plan)
+            if plan > last_plan:
+                last_plan = plan
+        self._last_plan = last_plan
+        plan_ticks = self._plan_ticks
+        time_plan = self._time_plan
+        for plan, turn, tick in self.query.plan_ticks_dump():
+            plan_ticks[plan][turn].append(tick)
+            time_plan[plans[plan][0], turn, tick] = plan
 
     def __enter__(self):
         """Enable the use of the ``with`` keyword"""
@@ -618,7 +675,7 @@ class ORM(object):
     def _set_branch(self, v):
         if self._planning:
             raise ValueError("Don't change branches while planning")
-        curbranch, curturn, curtick = self.btt()
+        curbranch, curturn, curtick = self._btt()
         if curbranch == v:
             self._otick = self._turn_end_plan[curbranch, curturn]
             return
@@ -635,7 +692,8 @@ class ORM(object):
                         rv=parturn
                     )
                 )
-        if v not in self._branches:
+        branch_is_new = v not in self._branches
+        if branch_is_new:
             # assumes the present turn in the parent branch has
             # been finalized.
             self.query.new_branch(v, curbranch, curturn, curtick)
@@ -644,6 +702,76 @@ class ORM(object):
             self._turn_end_plan[v, curturn] = self._turn_end[v, curturn] = curtick
         self._obranch = v
         self._otick = self._turn_end_plan[v, curturn]
+        if branch_is_new:
+            self._copy_plans(curbranch, curturn, curtick)
+
+    def _copy_plans(self, branch_from, turn_from, tick_from):
+        """Collect all plans that are active at the given time and copy them to the current branch"""
+        plan_ticks = self._plan_ticks
+        plan_ticks_uncommitted = self._plan_ticks_uncommitted
+        time_plan = self._time_plan
+        plans = self._plans
+        branch = self.branch
+        where_cached = self._where_cached
+        last_plan = self._last_plan
+        turn_end_plan = self._turn_end_plan
+        for plan_id in self._branches_plans[branch_from]:
+            _, start_turn, start_tick = plans[plan_id]
+            if start_turn > turn_from or (start_turn == turn_from and start_tick > tick_from):
+                continue
+            incremented = False
+            for turn, ticks in list(plan_ticks[plan_id].items()):
+                if turn < turn_from:
+                    continue
+                for tick in ticks:
+                    if turn == turn_from and tick < tick_from:
+                        continue
+                    if not incremented:
+                        self._last_plan = last_plan = last_plan + 1
+                        incremented = True
+                        plans[last_plan] = branch, turn, tick
+                    for cache in where_cached[branch_from, turn, tick]:
+                        data = cache.settings[branch_from][turn][tick]
+                        value = data[-1]
+                        key = data[:-1]
+                        args = key + (branch, turn, tick, value)
+                        if hasattr(cache, 'setdb'):
+                            cache.setdb(*args)
+                        cache.store(*args, planning=True)
+                        plan_ticks[last_plan][turn].append(tick)
+                        plan_ticks_uncommitted.append((last_plan, turn, tick))
+                        time_plan[branch, turn, tick] = last_plan
+                        turn_end_plan[branch, turn] = tick
+
+    def delete_plan(self, plan):
+        """Delete the portion of a plan that has yet to occur.
+
+        :arg plan: integer ID of a plan, as given by ``with self.plan() as plan:``
+
+        """
+        branch, turn, tick = self._btt()
+        to_delete = []
+        plan_ticks = self._plan_ticks[plan]
+        for trn, tcks in plan_ticks.items():  # might improve performance to use a WindowDict for plan_ticks
+            if turn == trn:
+                for tck in tcks:
+                    if tck >= tick:
+                        to_delete.append((trn, tck))
+            elif trn > turn:
+                to_delete.extend((trn, tck) for tck in tcks)
+        # Delete stuff that happened at contradicted times, and then delete the times from the plan
+        where_cached = self._where_cached
+        time_plan = self._time_plan
+        for trn, tck in to_delete:
+            for cache in where_cached[branch, trn, tck]:
+                cache.remove(branch, trn, tck)
+                if hasattr(cache, 'deldb'):
+                    cache.deldb(branch, trn, tck)
+            del where_cached[branch, trn, tck]
+            plan_ticks[trn].remove(tck)
+            if not plan_ticks[trn]:
+                del plan_ticks[trn]
+            del time_plan[branch, trn, tck]
 
     # easier to override things this way
     @property
@@ -677,8 +805,6 @@ class ORM(object):
                     "occurs before the start of "
                     "the branch {}".format(v, branch)
                 )
-        if not self._planning and v > turn_end:
-            self._branches[branch] = parent, turn_start, tick_start, v, tick
         self._otick = tick
         self._oturn = v
 
@@ -720,11 +846,11 @@ class ORM(object):
     def tick(self, v):
         self._set_tick(v)
 
-    def btt(self):
+    def _btt(self):
         """Return the branch, turn, and tick."""
         return self._obranch, self._oturn, self._otick
 
-    def nbtt(self):
+    def _nbtt(self):
         """Increment the tick and return branch, turn, tick
 
         Unless we're viewing the past, in which case raise HistoryError.
@@ -734,7 +860,7 @@ class ORM(object):
 
         """
         from .cache import HistoryError
-        branch, turn, tick = self.btt()
+        branch, turn, tick = self._btt()
         tick += 1
         if (branch, turn) in self._turn_end_plan:
             if tick > self._turn_end_plan[branch, turn]:
@@ -755,13 +881,14 @@ class ORM(object):
             raise HistoryError(
                 "You're in the past. Go to turn {}, tick {} to change things".format(turn_end, tick_end)
             )
-        if not self._planning:
-            if turn_end != turn:
+        if self._planning:
+            if (turn, tick) in self._plan_ticks[self._last_plan]:
                 raise HistoryError(
-                    "When advancing time outside of a plan, you can't skip turns. Go to turn {}".format(turn_end)
+                    "Trying to make a plan at {}, but that time already happened".format((branch, turn, tick))
                 )
-            self._branches[branch] = parent, turn_start, tick_start, turn_end, tick
-            self._turn_end[branch, turn] = tick
+            self._plan_ticks[self._last_plan][turn].append(tick)
+            self._plan_ticks_uncommitted.append((self._last_plan, turn, tick))
+            self._time_plan[branch, turn, tick] = self._last_plan
         self._otick = tick
         return branch, turn, tick
 
@@ -781,17 +908,19 @@ class ORM(object):
         set_turn = self.query.set_turn
         for (branch, turn), plan_end_tick in self._turn_end_plan.items():
             set_turn(branch, turn, turn_end[branch], plan_end_tick)
+        if self._plans_uncommitted:
+            self.query.plans_insert_many(self._plans_uncommitted)
+        if self._plan_ticks_uncommitted:
+            self.query.plan_ticks_insert_many(self._plan_ticks_uncommitted)
         self.query.commit()
+        self._plans_uncommitted = []
+        self._plan_ticks_uncommitted = []
 
     def close(self):
         """Write changes to database and close the connection"""
         self.commit()
         self.query.close()
 
-    def initdb(self):
-        """Alias of ``self.query.initdb``"""
-        self.query.initdb()
-
     def _init_graph(self, name, type_s='Graph'):
         if self.query.have_graph(name):
             raise GraphNameError("Already have a graph by that name")
@@ -803,6 +932,9 @@ class ORM(object):
         """Return a new instance of type Graph, initialized with the given
         data if provided.
 
+        :arg name: a name for the graph
+        :arg data: dictionary or NetworkX graph object providing initial state
+
         """
         self._init_graph(name, 'Graph')
         g = Graph(self, name, data, **attr)
@@ -813,6 +945,9 @@ class ORM(object):
         """Return a new instance of type DiGraph, initialized with the given
         data if provided.
 
+        :arg name: a name for the graph
+        :arg data: dictionary or NetworkX graph object providing initial state
+
         """
         self._init_graph(name, 'DiGraph')
         dg = DiGraph(self, name, data, **attr)
@@ -823,6 +958,9 @@ class ORM(object):
         """Return a new instance of type MultiGraph, initialized with the given
         data if provided.
 
+        :arg name: a name for the graph
+        :arg data: dictionary or NetworkX graph object providing initial state
+
         """
         self._init_graph(name, 'MultiGraph')
         mg = MultiGraph(self, name, data, **attr)
@@ -833,6 +971,9 @@ class ORM(object):
         """Return a new instance of type MultiDiGraph, initialized with the given
         data if provided.
 
+        :arg name: a name for the graph
+        :arg data: dictionary or NetworkX graph object providing initial state
+
         """
         self._init_graph(name, 'MultiDiGraph')
         mdg = MultiDiGraph(self, name, data, **attr)
@@ -844,6 +985,8 @@ class ORM(object):
         ``new_digraph``, ``new_multigraph``, or
         ``new_multidigraph``
 
+        :arg name: name of an existing graph
+
         """
         if name in self._graph_objs:
             return self._graph_objs[name]
@@ -863,7 +1006,11 @@ class ORM(object):
         return g
 
     def del_graph(self, name):
-        """Remove all traces of a graph's existence from the database"""
+        """Remove all traces of a graph's existence from the database
+
+        :arg name: name of an existing graph
+
+        """
         # make sure the graph exists before deleting anything
         self.get_graph(name)
         self.query.del_graph(name)
@@ -876,7 +1023,7 @@ class ORM(object):
         presently active and ending at 'trunk'), and the turn is the
         latest revision in the branch that matters.
 
-        Keyword ``stoptime`` may be a branch, in which case iteration will stop
+        :arg stoptime: This may be a branch, in which case iteration will stop
         instead of proceeding into that branch's parent; or it may be a triple,
         ``(branch, turn, tick)``, in which case iteration will stop instead of
         yielding any time before that. The tick may be ``None``, in which case
@@ -922,10 +1069,10 @@ class ORM(object):
                 yield child
 
     def _node_exists(self, character, node):
-        return self._nodes_cache.contains_entity(character, node, *self.btt())
+        return self._nodes_cache.contains_entity(character, node, *self._btt())
 
     def _exist_node(self, character, node, exist=True):
-        branch, turn, tick = self.nbtt()
+        branch, turn, tick = self._nbtt()
         self.query.exist_node(
             character,
             node,
@@ -938,13 +1085,13 @@ class ORM(object):
 
     def _edge_exists(self, character, orig, dest, idx=0):
         return self._edges_cache.contains_entity(
-            character, orig, dest, idx, *self.btt()
+            character, orig, dest, idx, *self._btt()
         )
 
     def _exist_edge(
             self, character, orig, dest, idx=0, exist=True
     ):
-        branch, turn, tick = self.nbtt()
+        branch, turn, tick = self._nbtt()
         self.query.exist_edge(
             character,
             orig,
