@@ -16,7 +16,7 @@
 """allegedb's special implementations of the NetworkX graph objects"""
 import networkx
 from networkx.exception import NetworkXError
-from collections import defaultdict
+from collections import defaultdict, MutableMapping
 from .wrap import MutableMappingUnwrapper
 
 class EntityCollisionError(ValueError):
@@ -31,7 +31,7 @@ def getatt(attribute_name):
 
 def convert_to_networkx_graph(data, create_using=None, multigraph_input=False):
     """Convert an AllegedGraph to the corresponding NetworkX graph type."""
-    if isinstance(data, AllegedGraph):
+    if isinstance(data, Graph):
         result = networkx.convert.from_dict_of_dicts(
             data.adj,
             create_using=create_using,
@@ -106,7 +106,7 @@ class AbstractEntityMapping(AllegedMapping):
         raise NotImplementedError
 
     def _get_cache_now(self, key):
-        return self._get_cache(key, *self.db.btt())
+        return self._get_cache(key, *self.db._btt())
 
     def _cache_contains(self, key, branch, turn, tick):
         raise NotImplementedError
@@ -119,7 +119,7 @@ class AbstractEntityMapping(AllegedMapping):
         raise NotImplementedError
 
     def _set_cache_now(self, key, value):
-        branch, turn, tick = self.db.nbtt()
+        branch, turn, tick = self.db._nbtt()
         self._set_cache(key, branch, turn, tick, value)
 
     def _del_db(self, key, branch, turn, tick):
@@ -149,7 +149,7 @@ class AbstractEntityMapping(AllegedMapping):
         return wrapval(self._get_cache_now(key))
 
     def __contains__(self, item):
-        return self._cache_contains(item, *self.db.btt())
+        return self._cache_contains(item, *self.db._btt())
 
     def __setitem__(self, key, value):
         """Set key=value at the present branch and revision"""
@@ -157,7 +157,7 @@ class AbstractEntityMapping(AllegedMapping):
             raise ValueError(
                 "allegedb uses None to indicate that a key's been deleted"
             )
-        branch, turn, tick = self.db.nbtt()
+        branch, turn, tick = self.db._nbtt()
         try:
             if self._get_cache(key, branch, turn, tick) != value:
                 self._set_cache(key, branch, turn, tick, value)
@@ -167,7 +167,7 @@ class AbstractEntityMapping(AllegedMapping):
         self.send(self, key=key, value=value)
 
     def __delitem__(self, key):
-        branch, turn, tick = self.db.nbtt()
+        branch, turn, tick = self.db._nbtt()
         self._del_cache(key, branch, turn, tick)
         self._del_db(key, branch, turn, tick)
         self.send(self, key=key, value=None)
@@ -175,29 +175,41 @@ class AbstractEntityMapping(AllegedMapping):
 
 class GraphMapping(AbstractEntityMapping):
     """Mapping for graph attributes"""
-    __slots__ = ('graph',)
-
-    db = getatt('graph.db')
+    __slots__ = ('graph', 'db', '_iter_stuff', '_cache_contains_stuff',
+                 '_len_stuff', '_get_stuff', '_set_db_stuff',
+                 '_set_cache_stuff', '_del_db_stuff', '_get_cache_stuff')
 
     def __init__(self, graph):
         super().__init__()
         self.graph = graph
+        self.db = db = graph.db
+        btt = db._btt
+        graph_val_cache = db._graph_val_cache
+        graphn = graph.name
+        self._iter_stuff = (graph_val_cache.iter_entity_keys, graphn, btt)
+        self._cache_contains_stuff = (graph_val_cache.contains_key, graphn)
+        self._len_stuff = (graph_val_cache.count_entities, graphn, btt)
+        self._get_stuff = (self._get_cache, btt)
+        graph_val_set = db.query.graph_val_set
+        self._set_db_stuff = (graph_val_set, graphn)
+        self._set_cache_stuff = (graph_val_cache.store, graphn)
+        self._del_db_stuff = (graph_val_set, graphn)
+        self._get_cache_stuff = (graph_val_cache.retrieve, graphn)
 
     def __iter__(self):
+        iter_entity_keys, graphn, btt = self._iter_stuff
         yield 'name'
-        yield from self.db._graph_val_cache.iter_entity_keys(
-            self.graph.name, *self.db.btt()
-        )
+        yield from iter_entity_keys(graphn, *btt())
 
     def _cache_contains(self, key, branch, turn, tick):
-        return self.db._graph_val_cache.contains_key(
-            self.graph.name, key, branch, turn, tick
+        contains_key, graphn = self._cache_contains_stuff
+        return contains_key(
+            graphn, key, branch, turn, tick
         )
 
     def __len__(self):
-        return 1 + self.db._graph_val_cache.count_entities(
-            self.graph.name, *self.db.btt()
-        )
+        count_entities, graphn, btt = self._len_stuff
+        return 1 + count_entities(graphn, *btt())
 
     def __getitem__(self, item):
         if item == 'name':
@@ -210,48 +222,53 @@ class GraphMapping(AbstractEntityMapping):
         super().__setitem__(key, value)
 
     def _get_cache(self, key, branch, turn, tick):
-        return self.db._graph_val_cache.retrieve(
-            self.graph.name, key, branch, turn, tick
+        retrieve, graphn = self._get_cache_stuff
+        return retrieve(
+            graphn, key, branch, turn, tick
         )
 
     def _get(self, key):
-        return self._get_cache(key, *self.db.btt())
+        get_cache, btt = self._get_stuff
+        return get_cache(key, *btt())
 
     def _set_db(self, key, branch, turn, tick, value):
-        self.db.query.graph_val_set(
-            self.graph.name,
+        graph_val_set, graphn = self._set_db_stuff
+        graph_val_set(
+            graphn,
             key,
             branch, turn, tick,
             value
         )
 
     def _set_cache(self, key, branch, turn, tick, value):
-        self.db._graph_val_cache.store(
-            self.graph.name, key, branch, turn, tick, value
+        store, graphn = self._set_cache_stuff
+        store(
+            graphn, key, branch, turn, tick, value
         )
 
     def _del_db(self, key, branch, turn, tick):
-        self.db.query.graph_val_del(
-            self.graph.name,
+        graph_val_set, graphn = self._del_db_stuff
+        graph_val_set(
+            graphn,
             key,
-            branch, turn, tick
+            branch, turn, tick, None
         )
 
     def clear(self):
-        keys = list(self.keys())
+        keys = set(self.keys())
         keys.remove('name')
         for k in keys:
             del self[k]
 
     def unwrap(self):
-        return {k: v.unwrap() if hasattr(v, 'unwrap') else v for (k, v) in self.items()}
+        return {k: v.unwrap() if hasattr(v, 'unwrap') and not hasattr(v, 'no_unwrap') else v for (k, v) in self.items()}
 
 
 class Node(AbstractEntityMapping):
     """Mapping for node attributes"""
-    __slots__ = ('graph', 'node', '__weakref__')
-
-    db = getatt('graph.db')
+    __slots__ = ('graph', 'node', 'db', '__weakref__', '_iter_stuff',
+                 '_cache_contains_stuff', '_len_stuff', '_get_cache_stuff',
+                 '_set_db_stuff', '_set_cache_stuff')
 
     def __new__(cls, graph, node):
         if (graph.name, node) in graph.db._node_objs:
@@ -267,6 +284,16 @@ class Node(AbstractEntityMapping):
         super().__init__()
         self.graph = graph
         self.node = node
+        self.db = db = graph.db
+        node_val_cache = db._node_val_cache
+        graphn = graph.name
+        btt = db._btt
+        self._iter_stuff = (node_val_cache.iter_entity_keys, graphn, node, btt)
+        self._cache_contains_stuff = (node_val_cache.contains_key, graphn, node)
+        self._len_stuff = (node_val_cache.count_entity_keys, graphn, node, btt)
+        self._get_cache_stuff = (node_val_cache.retrieve, graphn, node)
+        self._set_db_stuff = (db.query.node_val_set, graphn, node)
+        self._set_cache_stuff = (db._node_val_cache.store, graphn, node)
 
     def __repr__(self):
         return "{}(graph={}, node={})".format(self.__class__.__name__, self.graph, self.node)
@@ -275,38 +302,40 @@ class Node(AbstractEntityMapping):
         return "{}(graph={}, node={}, data={})".format(self.__class__.__name__, self.graph, self.node, repr(dict(self)))
 
     def __iter__(self):
-        return self.db._node_val_cache.iter_entity_keys(
-            self.graph.name, self.node, *self.db.btt()
-        )
+        iter_entity_keys, graphn, node, btt = self._iter_stuff
+        return iter_entity_keys(graphn, node, *btt())
 
     def _cache_contains(self, key, branch, turn, tick):
-        return self.db._node_val_cache.contains_key(
-            self.graph.name, self.node, key, branch, turn, tick
+        contains_key, graphn, node = self._cache_contains_stuff
+        return contains_key(
+            graphn, node, key, branch, turn, tick
         )
 
     def __len__(self):
-        return self.db._node_val_cache.count_entity_keys(
-            self.graph.name, self.node, *self.db.btt()
-        )
+        count_entity_keys, graphn, node, btt = self._len_stuff
+        return count_entity_keys(graphn, node, *btt())
 
     def _get_cache(self, key, branch, turn, tick):
-        return self.db._node_val_cache.retrieve(
-            self.graph.name, self.node, key, branch, turn, tick
+        retrieve, graphn, node = self._get_cache_stuff
+        return retrieve(
+            graphn, node, key, branch, turn, tick
         )
 
     def _set_db(self, key, branch, turn, tick, value):
-        self.db.query.node_val_set(
-            self.graph.name,
-            self.node,
+        node_val_set, graphn, node = self._set_db_stuff
+        node_val_set(
+            graphn,
+            node,
             key,
             branch, turn, tick,
             value
         )
 
     def _set_cache(self, key, branch, turn, tick, value):
-        self.db._node_val_cache.store(
-            self.graph.name,
-            self.node,
+        store, graphn, node = self._set_cache_stuff
+        store(
+            graphn,
+            node,
             key,
             branch, turn, tick,
             value
@@ -315,9 +344,9 @@ class Node(AbstractEntityMapping):
 
 class Edge(AbstractEntityMapping):
     """Mapping for edge attributes"""
-    __slots__ = ('graph', 'orig', 'dest', 'idx', '__weakref__')
-
-    db = getatt('graph.db')
+    __slots__ = ('graph', 'orig', 'dest', 'idx', 'db', '__weakref__',
+                 '_iter_stuff', '_cache_contains_stuff', '_len_stuff',
+                 '_get_cache_stuff', '_set_db_stuff', '_set_cache_stuff')
 
     def __new__(cls, graph, orig, dest, idx=0):
         if (graph.name, orig, dest, idx) in graph.db._edge_objs:
@@ -339,9 +368,19 @@ class Edge(AbstractEntityMapping):
         """
         super().__init__()
         self.graph = graph
+        self.db = db = graph.db
         self.orig = orig
         self.dest = dest
         self.idx = idx
+        edge_val_cache = db._edge_val_cache
+        graphn = graph.name
+        btt = db._btt
+        self._iter_stuff = (edge_val_cache.iter_entity_keys, graphn, orig, dest, idx, btt)
+        self._cache_contains_stuff = (edge_val_cache.contains_key, graphn, orig, dest, idx)
+        self._len_stuff = (edge_val_cache.count_entity_keys, graphn, orig, dest, idx, btt)
+        self._get_cache_stuff = (edge_val_cache.retrieve, graphn, orig, dest, idx)
+        self._set_db_stuff = (db.query.edge_val_set, graphn, orig, dest, idx)
+        self._set_cache_stuff = (edge_val_cache.store, graphn, orig, dest, idx)
 
     def __repr__(self):
         if self.idx == 0:
@@ -363,55 +402,40 @@ class Edge(AbstractEntityMapping):
         )
 
     def __iter__(self):
-        return self.db._edge_val_cache.iter_entity_keys(
-            self.graph.name,
-            self.orig,
-            self.dest,
-            self.idx,
-            *self.db.btt()
-        )
+        iter_entity_keys, graphn, orig, dest, idx, btt = self._iter_stuff
+        return iter_entity_keys(graphn, orig, dest, idx, *btt())
 
     def _cache_contains(self, key, branch, turn, tick):
-        return self.db._edge_val_cache.contains_key(
-            self.graph.name, self.orig, self.dest, self.idx, key, branch, turn, tick
-        )
+        contains_key, graphn, orig, dest, idx = self._cache_contains_stuff
+        return contains_key(graphn, orig, dest, idx, key, branch, turn, tick)
 
     def __len__(self):
-        return self.db._edge_val_cache.count_entity_keys(
-            self.graph.name,
-            self.orig,
-            self.dest,
-            self.idx,
-            *self.db.btt()
-        )
+        count_entity_keys, graphn, orig, dest, idx, btt = self._len_stuff
+        return count_entity_keys(graphn, orig, dest, idx, *btt())
 
     def _get_cache(self, key, branch, turn, tick):
-        return self.db._edge_val_cache.retrieve(
-            self.graph.name,
-            self.orig,
-            self.dest,
-            self.idx,
-            key,
-            branch, turn, tick
-        )
+        retrieve, graphn, orig, dest, idx = self._get_cache_stuff
+        return retrieve(graphn, orig, dest, idx, key, branch, turn, tick)
 
     def _set_db(self, key, branch, turn, tick, value):
-        self.db.query.edge_val_set(
-            self.graph.name,
-            self.orig,
-            self.dest,
-            self.idx,
+        edge_val_set, graphn, orig, dest, idx = self._set_db_stuff
+        edge_val_set(
+            graphn,
+            orig,
+            dest,
+            idx,
             key,
             branch, turn, tick,
             value
         )
 
     def _set_cache(self, key, branch, turn, tick, value):
-        self.db._edge_val_cache.store(
-            self.graph.name,
-            self.orig,
-            self.dest,
-            self.idx,
+        store, graphn, orig, dest, idx = self._set_cache_stuff
+        store(
+            graphn,
+            orig,
+            dest,
+            idx,
             key,
             branch, turn, tick,
             value
@@ -423,6 +447,7 @@ class GraphNodeMapping(AllegedMapping):
     __slots__ = ('graph',)
 
     db = getatt('graph.db')
+    """Alias to ``self.graph.db``"""
 
     def __init__(self, graph):
         super().__init__()
@@ -431,7 +456,7 @@ class GraphNodeMapping(AllegedMapping):
     def __iter__(self):
         """Iterate over the names of the nodes"""
         return self.db._nodes_cache.iter_entities(
-            self.graph.name, *self.db.btt()
+            self.graph.name, *self.db._btt()
         )
 
     def __eq__(self, other):
@@ -443,9 +468,9 @@ class GraphNodeMapping(AllegedMapping):
         for k in self.keys():
             me = self[k]
             you = other[k]
-            if hasattr(me, 'unwrap'):
+            if hasattr(me, 'unwrap') and not hasattr(me, 'no_unwrap'):
                 me = me.unwrap()
-            if hasattr(you, 'unwrap'):
+            if hasattr(you, 'unwrap') and not hasattr(you, 'no_unwrap'):
                 you = you.unwrap()
             if me != you:
                 return False
@@ -455,13 +480,13 @@ class GraphNodeMapping(AllegedMapping):
     def __contains__(self, node):
         """Return whether the node exists presently"""
         return self.db._nodes_cache.contains_entity(
-            self.graph.name, node, *self.db.btt()
+            self.graph.name, node, *self.db._btt()
         )
 
     def __len__(self):
         """How many nodes exist right now?"""
         return self.db._nodes_cache.count_entities(
-            self.graph.name, *self.db.btt()
+            self.graph.name, *self.db._btt()
         )
 
     def __getitem__(self, node):
@@ -476,25 +501,24 @@ class GraphNodeMapping(AllegedMapping):
         is made with them, perhaps clearing out the one already there.
 
         """
-        branch, turn, tick = self.db.nbtt()
-        created = node not in self
-        n = self.db._get_node(self.graph, node)
+        created = False
+        db = self.db
+        graph = self.graph
+        gname = graph.name
+        if not db._node_exists(gname, node):
+            created = True
+            db._exist_node(gname, node, True)
+        n = db._get_node(graph, node)
         n.clear()
         n.update(dikt)
         if created:
-            self.db.query.exist_node(
-                self.graph.name,
-                node,
-                branch, turn, tick,
-                True
-            )
             self.send(self, node_name=node, exists=True)
 
     def __delitem__(self, node):
         """Indicate that the given node no longer exists"""
         if node not in self:
             raise KeyError("No such node")
-        branch, turn, tick = self.db.nbtt()
+        branch, turn, tick = self.db._nbtt()
         self.db.query.exist_node(
             self.graph.name,
             node,
@@ -547,6 +571,7 @@ class GraphEdgeMapping(AllegedMapping):
         return self._metacache[id(self)]
 
     db = getatt('graph.db')
+    """Alias to ``self.graph.db``"""
 
     def __init__(self, graph):
         super().__init__()
@@ -579,8 +604,12 @@ class AbstractSuccessors(GraphEdgeMapping):
     __slots__ = ('graph', 'container', 'orig')
 
     db = getatt('graph.db')
+    """Alias to ``self.graph.db``"""
     _metacache = defaultdict(dict)
 
+    def _order_nodes(self, node):
+        raise NotImplemented
+
     @property
     def _cache(self):
         return self._metacache[id(self)]
@@ -596,16 +625,17 @@ class AbstractSuccessors(GraphEdgeMapping):
         return self.db._edges_cache.iter_successors(
             self.graph.name,
             self.orig,
-            *self.db.btt()
+            *self.db._btt()
         )
 
     def __contains__(self, dest):
         """Is there an edge leading to ``dest`` at the moment?"""
+        orig, dest = self._order_nodes(dest)
         return self.db._edges_cache.has_successor(
             self.graph.name,
-            self.orig,
+            orig,
             dest,
-            *self.db.btt()
+            *self.db._btt()
         )
 
     def __len__(self):
@@ -613,28 +643,35 @@ class AbstractSuccessors(GraphEdgeMapping):
         return self.db._edges_cache.count_successors(
             self.graph.name,
             self.orig,
-            *self.db.btt()
+            *self.db._btt()
         )
 
     def _make_edge(self, dest):
-        return Edge(self.graph, self.orig, dest)
+        return Edge(self.graph, *self._order_nodes(dest))
 
     def __getitem__(self, dest):
         """Get the edge between my orig and the given node"""
         if dest not in self:
             raise KeyError("No edge {}->{}".format(self.orig, dest))
-        return self.db._get_edge(self.graph, self.orig, dest, 0)
+        orig, dest = self._order_nodes(dest)
+        return self.db._get_edge(self.graph, orig, dest, 0)
 
     def __setitem__(self, dest, value):
         """Set the edge between my orig and the given dest to the given
         value, a mapping.
 
         """
+        real_dest = dest
+        orig, dest = self._order_nodes(dest)
         created = dest not in self
-        branch, turn, tick = self.db.nbtt()
+        if orig not in self.graph.node:
+            self.graph.add_node(orig)
+        if dest not in self.graph.node:
+            self.graph.add_node(dest)
+        branch, turn, tick = self.db._nbtt()
         self.db.query.exist_edge(
             self.graph.name,
-            self.orig,
+            orig,
             dest,
             0,
             branch, turn, tick,
@@ -642,24 +679,25 @@ class AbstractSuccessors(GraphEdgeMapping):
         )
         self.db._edges_cache.store(
             self.graph.name,
-            self.orig,
+            orig,
             dest,
             0,
             branch, turn, tick,
             True
         )
-        e = self[dest]
+        e = self[real_dest]
         e.clear()
         e.update(value)
         if created:
-            self.send(self, orig=self.orig, dest=dest, idx=0, exists=True)
+            self.send(self, orig=orig, dest=dest, idx=0, exists=True)
 
     def __delitem__(self, dest):
         """Remove the edge between my orig and the given dest"""
-        branch, turn, tick = self.db.nbtt()
+        branch, turn, tick = self.db._nbtt()
+        orig, dest = self._order_nodes(dest)
         self.db.query.exist_edge(
             self.graph.name,
-            self.orig,
+            orig,
             dest,
             0,
             branch, turn, tick,
@@ -667,13 +705,17 @@ class AbstractSuccessors(GraphEdgeMapping):
         )
         self.db._edges_cache.store(
             self.graph.name,
-            self.orig,
+            orig,
             dest,
             0,
             branch, turn, tick,
             None
         )
-        self.send(self, orig=self.orig, dest=dest, idx=0, exists=False)
+        self.send(self, orig=orig, dest=dest, idx=0, exists=False)
+
+    def __repr__(self):
+        cls = self.__class__
+        return "<{}.{} object containing {}>".format(cls.__module__, cls.__name__, dict(self))
 
     def clear(self):
         """Delete every edge with origin at my orig"""
@@ -732,6 +774,13 @@ class GraphSuccessorsMapping(GraphEdgeMapping):
     def __contains__(self, key):
         return key in self.graph.node
 
+    def __repr__(self):
+        cls = self.__class__
+        return "<{}.{} object containing {}>".format(
+            cls.__module__, cls.__name__, {
+                k: dict(v) for (k, v) in self.items()
+        })
+
 
 class DiGraphSuccessorsMapping(GraphSuccessorsMapping):
     __slots__ = ('graph',)
@@ -777,9 +826,9 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
 
     def __delitem__(self, key):
         """Delete all edges ending at ``dest``"""
-        if key in self._cache:
-            self._cache[key].clear()
-            del self._cache[key]
+        it = self[key]
+        it.clear()
+        del self._cache[key]
         self.send(self, key=key, val=None)
 
     def __iter__(self):
@@ -805,7 +854,7 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
             return self.db._edges_cache.iter_predecessors(
                 self.graph.name,
                 self.dest,
-                *self.db.btt()
+                *self.db._btt()
             )
 
         def __contains__(self, orig):
@@ -814,7 +863,7 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
                 self.graph.name,
                 self.dest,
                 orig,
-                *self.db.btt()
+                *self.db._btt()
             )
 
         def __len__(self):
@@ -822,7 +871,7 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
             return self.db._edges_cache.count_predecessors(
                 self.graph.name,
                 self.dest,
-                *self.db.btt()
+                *self.db._btt()
             )
 
         def _make_edge(self, orig):
@@ -837,7 +886,7 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
             given node to mine.
 
             """
-            branch, turn, tick = self.db.nbtt()
+            branch, turn, tick = self.db._nbtt()
             try:
                 e = self[orig]
                 e.clear()
@@ -865,7 +914,7 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
 
         def __delitem__(self, orig):
             """Unset the existence of the edge from the given node to mine"""
-            branch, turn, tick = self.db.nbtt()
+            branch, turn, tick = self.db._nbtt()
             if 'Multi' in self.graph.__class__.__name__:
                 for idx in self[orig]:
                     self.db.query.exist_edge(
@@ -907,11 +956,16 @@ class DiGraphPredecessorsMapping(GraphEdgeMapping):
             self.send(self, key=orig, val=None)
 
 
-class MultiEdges(GraphEdgeMapping):
+class AbstractMultiEdges(GraphEdgeMapping):
     """Mapping of Edges between two nodes"""
     __slots__ = ('graph', 'orig', 'dest')
 
     db = getatt('graph.db')
+    """Alias to ``self.graph.db``"""
+
+    def _order_nodes(self):
+        """Swap my orig and dest if desired"""
+        raise NotImplemented
 
     def __init__(self, graph, orig, dest):
         super().__init__(graph)
@@ -919,9 +973,10 @@ class MultiEdges(GraphEdgeMapping):
         self.dest = dest
 
     def __iter__(self):
+        orig, dest = self._order_nodes()
         return self.db._edges_cache.iter_keys(
-            self.graph.name, self.orig, self.dest,
-            *self.db.btt()
+            self.graph.name, orig, dest,
+            *self.db._btt()
         )
 
     def __len__(self):
@@ -932,13 +987,15 @@ class MultiEdges(GraphEdgeMapping):
         return n
 
     def __contains__(self, i):
+        orig, dest = self._order_nodes()
         return self.db._edges_cache.contains_key(
-            self.graph.name, self.orig, self.dest, i,
-            *self.db.btt()
+            self.graph.name, orig, dest, i,
+            *self.db._btt()
         )
 
     def _getedge(self, idx):
-        return self.db._get_edge(self.graph, self.orig, self.dest, idx)
+        orig, dest = self._order_nodes()
+        return self.db._get_edge(self.graph, orig, dest, idx)
 
     def __getitem__(self, idx):
         """Get an Edge with a particular index, if it exists at the present
@@ -954,41 +1011,46 @@ class MultiEdges(GraphEdgeMapping):
         Edge first, if necessary.
 
         """
-        branch, turn, tick = self.db.nbtt()
+        orig, dest = self._order_nodes()
         created = idx not in self
+        if orig not in self.graph.node:
+            self.graph.add_node(orig)
+        if dest not in self.graph.node:
+            self.graph.add_node(dest)
+        branch, turn, tick = self.db._nbtt()
         self.db.query.exist_edge(
             self.graph.name,
-            self.orig,
-            self.dest,
+            orig,
+            dest,
             idx,
             branch, turn, tick,
             True
         )
+        self.db._edges_cache.store(
+            self.graph.name, orig, dest, idx,
+            branch, turn, tick, True
+        )
         e = self._getedge(idx)
         e.clear()
         e.update(val)
-        self.db._edges_cache.store(
-            self.graph.name, self.orig, self.dest, idx,
-            branch, turn, tick, True
-        )
         if created:
-            self.send(self, orig=self.orig, dest=self.dest, idx=idx, exists=True)
+            self.send(self, orig=orig, dest=dest, idx=idx, exists=True)
 
     def __delitem__(self, idx):
         """Delete the edge at a particular index"""
-        branch, turn, tick = self.db.btt()
+        branch, turn, tick = self.db._btt()
+        orig, dest = self._order_nodes()
         tick += 1
         e = self._getedge(idx)
-        if not e.exists:
-            raise KeyError("No edge at that index")
         e.clear()
-        del self._cache[idx]
+        if idx in self._cache:
+            del self._cache[idx]
         self.db._edges_cache.store(
-            self.graph.name, self.orig, self.dest, idx,
+            self.graph.name, orig, dest, idx,
             branch, turn, tick, None
         )
         self.db.tick = tick
-        self.send(self, orig=self.orig, dest=self.dest, idx=idx, exists=False)
+        self.send(self, orig=orig, dest=dest, idx=idx, exists=False)
 
     def clear(self):
         """Delete all edges between these nodes"""
@@ -1032,10 +1094,23 @@ class MultiGraphSuccessorsMapping(GraphSuccessorsMapping):
         """Edges succeeding a given node in a multigraph"""
         __slots__ = ('graph', 'container', 'orig', '_multedge')
 
+        class MultiEdges(AbstractMultiEdges):
+            def _order_nodes(self):
+                if self.dest < self.orig:
+                    return (self.dest, self.orig)
+                else:
+                    return (self.orig, self.dest)
+
         def __init__(self, container, orig):
             super().__init__(container, orig)
             self._multedge = {}
 
+        def __contains__(self, item):
+            orig, dest = self._order_nodes(item)
+            return self.db._edges_cache.has_successor(
+                self.graph.name, orig, dest, *self.db._btt()
+            )
+
         def _order_nodes(self, dest):
             if dest < self.orig:
                 return(dest, self.orig)
@@ -1044,7 +1119,7 @@ class MultiGraphSuccessorsMapping(GraphSuccessorsMapping):
 
         def _get_multedge(self, dest):
             if dest not in self._multedge:
-                self._multedge[dest] = MultiEdges(
+                self._multedge[dest] = self.MultiEdges(
                     self.graph, *self._order_nodes(dest)
                 )
             return self._multedge[dest]
@@ -1060,7 +1135,10 @@ class MultiGraphSuccessorsMapping(GraphSuccessorsMapping):
             between my ``orig`` and the given ``dest``
 
             """
-            self[dest].update(val)
+            it = self[dest]
+            it.clear()
+            it.update(val)
+            assert dest in self
             self.send(self, key=dest, val=val)
 
         def __delitem__(self, dest):
@@ -1078,9 +1156,13 @@ class MultiDiGraphPredecessorsMapping(DiGraphPredecessorsMapping):
         """Predecessor edges from a given node"""
         __slots__ = ('graph', 'container', 'dest')
 
+        class MultiEdges(AbstractMultiEdges):
+            def _order_nodes(self):
+                return self.orig, self.dest
+
         def __getitem__(self, orig):
             """Get MultiEdges"""
-            return MultiEdges(self.graph, orig, self.dest)
+            return self.MultiEdges(self.graph, orig, self.dest)
 
         def __setitem__(self, orig, val):
             self[orig].update(val)
@@ -1096,19 +1178,25 @@ class MultiDiGraphSuccessorsMapping(MultiGraphSuccessorsMapping):
 
     class Successors(MultiGraphSuccessorsMapping.Successors):
         __slots__ = ('graph', 'container', 'orig', '_multedge')
+
+        class MultiEdges(AbstractMultiEdges):
+            def _order_nodes(self):
+                return self.orig, self.dest
+
         def _order_nodes(self, dest):
             return self.orig, dest
 
 
-class AllegedGraph(object):
-    """Class giving the graphs those methods they share in
-    common.
+class Graph(networkx.Graph):
+    """A version of the networkx.Graph class that stores its state in a
+    database.
 
     """
     _succs = {}
     _statmaps = {}
     graph_map_cls = GraphMapping
     node_map_cls = GraphNodeMapping
+    adj_cls = GraphSuccessorsMapping
 
     def __new__(cls, db, name, data=None, **attr):
         if name in db._graph_objs:
@@ -1118,7 +1206,7 @@ class AllegedGraph(object):
                     name, type(ret)
                 ))
             return ret
-        return super(AllegedGraph, cls).__new__(cls)
+        return super(Graph, cls).__new__(cls)
 
     def __init__(self, db, name, data=None, **attr):
         self._name = name
@@ -1223,16 +1311,16 @@ class AllegedGraph(object):
         self.node.clear()
         self.graph.clear()
 
-
-class Graph(AllegedGraph, networkx.Graph):
-    """A version of the networkx.Graph class that stores its state in a
-    database.
-
-    """
-    adj_cls = GraphSuccessorsMapping
+    def add_node(self, node_for_adding, **attr):
+        if node_for_adding not in self._succ:
+            self._succ[node_for_adding] = self.adjlist_inner_dict_factory()
+            if hasattr(self, 'pred_cls'):
+                self._pred[node_for_adding] = self.adjlist_inner_dict_factory()
+            self._node[node_for_adding] = self.node_dict_factory()
+        self._node[node_for_adding].update(attr)
 
 
-class DiGraph(AllegedGraph, networkx.DiGraph):
+class DiGraph(Graph, networkx.DiGraph):
     """A version of the networkx.DiGraph class that stores its state in a
     database.
 
@@ -1275,12 +1363,16 @@ class DiGraph(AllegedGraph, networkx.DiGraph):
                 raise NetworkXError(
                     "The attr_dict argument must be a dictionary."
                 )
-        datadict = self.adj[u].get(v, {})
-        datadict.update(attr_dict)
         if u not in self.node:
             self.node[u] = {}
         if v not in self.node:
             self.node[v] = {}
+        if u in self.adj:
+            datadict = self.adj[u].get(v, {})
+        else:
+            self.adj[u] = {v: {}}
+            datadict = self.adj[u][v]
+        datadict.update(attr_dict)
         self.succ[u][v] = datadict
         assert u in self.succ, "Failed to add edge {u}->{v} ({u} not in successors)".format(u=u, v=v)
         assert v in self.succ[u], "Failed to add edge {u}->{v} ({v} not in succ[{u}])".format(u=u, v=v)
@@ -1320,7 +1412,7 @@ class DiGraph(AllegedGraph, networkx.DiGraph):
             assert(v in self.succ[u])
 
 
-class MultiGraph(AllegedGraph, networkx.MultiGraph):
+class MultiGraph(Graph, networkx.MultiGraph):
     """A version of the networkx.MultiGraph class that stores its state in a
     database.
 
@@ -1328,7 +1420,7 @@ class MultiGraph(AllegedGraph, networkx.MultiGraph):
     adj_cls = MultiGraphSuccessorsMapping
 
 
-class MultiDiGraph(AllegedGraph, networkx.MultiDiGraph):
+class MultiDiGraph(Graph, networkx.MultiDiGraph):
     """A version of the networkx.MultiDiGraph class that stores its state in a
     database.
 
@@ -1405,4 +1497,29 @@ class MultiDiGraph(AllegedGraph, networkx.MultiDiGraph):
             self.succ[u][v] = keydict
         return key
 
-pass
+
+class GraphsMapping(MutableMapping):
+    def __init__(self, orm):
+        self.orm = orm
+
+    def __iter__(self):
+        return iter(self.orm._graph_objs)
+
+    def __len__(self):
+        return len(self.orm._graph_objs)
+
+    def __getitem__(self, item):
+        return self.orm.get_graph(item)
+
+    def __setitem__(self, key, value):
+        if isinstance(value, networkx.MultiDiGraph):
+            self.orm.new_multidigraph(key, data=value)
+        elif isinstance(value, networkx.DiGraph):
+            self.orm.new_digraph(key, data=value)
+        elif isinstance(value, networkx.MultiGraph):
+            self.orm.new_multigraph(key, data=value)
+        else:
+            self.orm.new_graph(key, data=value)
+
+    def __delitem__(self, key):
+        self.orm.del_graph(key)
