@@ -229,8 +229,9 @@ class QueryEngine(object):
 
     def global_items(self):
         """Iterate over (key, value) pairs in the ``globals`` table."""
+        unpack = self.unpack
         for (k, v) in self.sql('global_dump'):
-            yield (self.unpack(k), self.unpack(v))
+            yield (unpack(k), unpack(v))
 
     def global_set(self, key, value):
         """Set ``key`` to ``value`` globally (not at any particular branch or
@@ -282,14 +283,15 @@ class QueryEngine(object):
     def graph_val_dump(self):
         """Yield the entire contents of the graph_val table."""
         self._flush_graph_val()
+        unpack = self.unpack
         for (graph, key, branch, turn, tick, value) in self.sql('graph_val_dump'):
             yield (
-                self.unpack(graph),
-                self.unpack(key),
+                unpack(graph),
+                unpack(key),
                 branch,
                 turn,
                 tick,
-                self.unpack(value)
+                unpack(value)
             )
 
     def _flush_graph_val(self):
@@ -320,9 +322,10 @@ class QueryEngine(object):
         graph, key, value = map(self.pack, (graph, key, value))
         self._graphvals2set.append((graph, key, branch, turn, tick, value))
 
-    def graph_val_del(self, graph, key, branch, turn, tick):
-        """Indicate that the key is unset."""
-        self.graph_val_set(graph, key, branch, turn, tick, None)
+    def graph_val_del_time(self, branch, turn, tick):
+        self._flush_graph_val()
+        self.sql('graph_val_del_time', branch, turn, tick)
+        self._btts.discard((branch, turn, tick))
 
     def graphs_types(self):
         for (graph, typ) in self.sql('graphs_types'):
@@ -355,13 +358,19 @@ class QueryEngine(object):
         self._btts.add((branch, turn, tick))
         self._nodes2set.append((self.pack(graph), self.pack(node), branch, turn, tick, extant))
 
+    def nodes_del_time(self, branch, turn, tick):
+        self._flush_nodes()
+        self.sql('nodes_del_time', branch, turn, tick)
+        self._btts.discard((branch, turn, tick))
+
     def nodes_dump(self):
         """Dump the entire contents of the nodes table."""
         self._flush_nodes()
+        unpack = self.unpack
         for (graph, node, branch, turn,tick, extant) in self.sql('nodes_dump'):
             yield (
-                self.unpack(graph),
-                self.unpack(node),
+                unpack(graph),
+                unpack(node),
                 branch,
                 turn,
                 tick,
@@ -371,17 +380,18 @@ class QueryEngine(object):
     def node_val_dump(self):
         """Yield the entire contents of the node_val table."""
         self._flush_node_val()
+        unpack = self.unpack
         for (
                 graph, node, key, branch, turn, tick, value
         ) in self.sql('node_val_dump'):
             yield (
-                self.unpack(graph),
-                self.unpack(node),
-                self.unpack(key),
+                unpack(graph),
+                unpack(node),
+                unpack(key),
                 branch,
                 turn,
                 tick,
-                self.unpack(value)
+                unpack(value)
             )
 
     def _flush_node_val(self):
@@ -414,20 +424,22 @@ class QueryEngine(object):
         graph, node, key, value = map(self.pack, (graph, node, key, value))
         self._nodevals2set.append((graph, node, key, branch, turn, tick, value))
 
-    def node_val_del(self, graph, node, key, branch, turn, tick):
-        """Delete a key from a node at a specific branch and revision"""
-        self.node_val_set(graph, node, key, branch, turn, tick, None)
+    def node_val_del_time(self, branch, turn, tick):
+        self._flush_node_val()
+        self.sql('node_val_del_time', branch, turn, tick)
+        self._btts.discard((branch, turn, tick))
 
     def edges_dump(self):
         """Dump the entire contents of the edges table."""
         self._flush_edges()
+        unpack = self.unpack
         for (
                 graph, orig, dest, idx, branch, turn, tick, extant
         ) in self.sql('edges_dump'):
             yield (
-                self.unpack(graph),
-                self.unpack(orig),
-                self.unpack(dest),
+                unpack(graph),
+                unpack(orig),
+                unpack(dest),
                 idx,
                 branch,
                 turn,
@@ -466,22 +478,28 @@ class QueryEngine(object):
         graph, orig, dest = map(self.pack, (graph, orig, dest))
         self._edges2set.append((graph, orig, dest, idx, branch, turn, tick, extant))
 
+    def edges_del_time(self, branch, turn, tick):
+        self._flush_edges()
+        self.sql('edges_del_time', branch, turn, tick)
+        self._btts.discard((branch, turn, tick))
+
     def edge_val_dump(self):
         """Yield the entire contents of the edge_val table."""
         self._flush_edge_val()
+        unpack = self.unpack
         for (
                 graph, orig, dest, idx, key, branch, turn, tick, value
         ) in self.sql('edge_val_dump'):
             yield (
-                self.unpack(graph),
-                self.unpack(orig),
-                self.unpack(dest),
+                unpack(graph),
+                unpack(orig),
+                unpack(dest),
                 idx,
-                self.unpack(key),
+                unpack(key),
                 branch,
                 turn,
                 tick,
-                self.unpack(value)
+                unpack(value)
             )
 
     def _flush_edge_val(self):
@@ -515,12 +533,28 @@ class QueryEngine(object):
             (graph, orig, dest, idx, key, branch, turn, tick, value)
         )
 
-    def edge_val_del(self, graph, orig, dest, idx, key, branch, turn, tick):
-        """Declare that the key no longer applies to this edge, as of this
-        branch and revision.
+    def edge_val_del_time(self, branch, turn, tick):
+        self._flush_edge_val()
+        self.sql('edge_val_del_time', branch, turn, tick)
+        self._btts.discard((branch, turn, tick))
 
-        """
-        self.edge_val_set(graph, orig, dest, idx, key, branch, turn, tick, None)
+    def plans_dump(self):
+        return self.sql('plans_dump')
+
+    def plans_insert(self, plan_id, branch, turn, tick):
+        return self.sql('plans_insert', plan_id, branch, turn, tick)
+
+    def plans_insert_many(self, many):
+        return self.sqlmany('plans_insert', *many)
+
+    def plan_ticks_insert(self, plan_id, turn, tick):
+        return self.sql('plan_ticks_insert', plan_id, turn, tick)
+
+    def plan_ticks_insert_many(self, many):
+        return self.sqlmany('plan_ticks_insert', *many)
+
+    def plan_ticks_dump(self):
+        return self.sql('plan_ticks_dump')
 
     def initdb(self):
         """Create tables and indices as needed."""
@@ -543,39 +577,23 @@ class QueryEngine(object):
             self.globl['turn'] = 0
         if 'tick' not in self.globl:
             self.globl['tick'] = 0
-        try:
-            cursor.execute('SELECT * FROM branches;')
-        except OperationalError:
-            cursor.execute(self.strings['create_branches'])
-        try:
-            cursor.execute('SELECT * FROM turns;')
-        except OperationalError:
-            cursor.execute(self.strings['create_turns'])
-        try:
-            cursor.execute('SELECT * FROM graphs;')
-        except OperationalError:
-            cursor.execute(self.strings['create_graphs'])
-        try:
-            cursor.execute('SELECT * FROM graph_val;')
-        except OperationalError:
-            cursor.execute(self.strings['create_graph_val'])
-        try:
-            cursor.execute('SELECT * FROM nodes;')
-        except OperationalError:
-            cursor.execute(self.strings['create_nodes'])
-
-        try:
-            cursor.execute('SELECT * FROM node_val;')
-        except OperationalError:
-            cursor.execute(self.strings['create_node_val'])
-        try:
-            cursor.execute('SELECT * FROM edges;')
-        except OperationalError:
-            cursor.execute(self.strings['create_edges'])
-        try:
-            cursor.execute('SELECT * FROM edge_val;')
-        except OperationalError:
-            cursor.execute(self.strings['create_edge_val'])
+        strings = self.strings
+        for table in (
+            'branches',
+            'turns',
+            'graphs',
+            'graph_val',
+            'nodes',
+            'node_val',
+            'edges',
+            'edge_val',
+            'plans',
+            'plan_ticks'
+        ):
+            try:
+                cursor.execute('SELECT * FROM ' + table + ';')
+            except OperationalError:
+                cursor.execute(strings['create_' + table])
 
     def flush(self):
         """Put all pending changes into the SQL transaction."""
