@@ -22,7 +22,22 @@ of the same key and neighboring ones repeatedly and in sequence.
 
 """
 from collections import deque, Mapping, MutableMapping, KeysView, ItemsView, ValuesView
-from operator import itemgetter
+from operator import itemgetter, lt, le
+from itertools import chain
+try:
+    import cython
+except ImportError:
+    class cython:
+        def locals(**kwargs):
+            def passthru(fun):
+                return fun
+            return passthru
+        cfunc = locals
+        int = None
+        bint = None
+
+get0 = itemgetter(0)
+get1 = itemgetter(1)
 
 # TODO: cancel changes that would put something back to where it was at the start
 # This will complicate the update_window functions though, and I don't think it'll
@@ -39,7 +54,7 @@ def update_window(turn_from, tick_from, turn_to, tick_to, updfun, branchd):
             for past_state in branchd[midturn][:]:
                 updfun(*past_state)
     if turn_to in branchd:
-        for past_state in branchd[turn_to][:tick_to]:
+        for past_state in branchd[turn_to][:tick_to+1]:
             updfun(*past_state)
 
 
@@ -70,8 +85,8 @@ def within_history(rev, windowdict):
     if not windowdict:
         return False
     begin = windowdict._past[0][0] if windowdict._past else \
-            windowdict._future[0][0]
-    end = windowdict._future[-1][0] if windowdict._future else \
+            windowdict._future[-1][0]
+    end = windowdict._future[0][0] if windowdict._future else \
           windowdict._past[-1][0]
     return begin <= rev <= end
 
@@ -79,215 +94,188 @@ def within_history(rev, windowdict):
 class WindowDictKeysView(KeysView):
     """Look through all the keys a WindowDict contains."""
     def __contains__(self, rev):
-        if not within_history(rev, self._mapping):
-            return False
-        for mrev, mv in self._mapping._past:
-            if mrev == rev:
-                return True
-        for mrev, mv in self._mapping._future:
-            if mrev == rev:
-                return True
-        return False
+        return rev in self._mapping._keys
 
     def __iter__(self):
-        if self._mapping._past:
-            for rev, v in self._mapping._past:
-                yield rev
-        if self._mapping._future:
-            for rev, v in self._mapping._future:
-                yield rev
+        past = self._mapping._past
+        future = self._mapping._future
+        if past:
+            yield from map(get0, past)
+        if future:
+            yield from map(get0, reversed(future))
 
 
 class WindowDictItemsView(ItemsView):
     """Look through everything a WindowDict contains."""
     def __contains__(self, item):
         (rev, v) = item
-        if not within_history(rev, self._mapping):
+        mapp = self._mapping
+        if not within_history(rev, mapp):
             return False
-        for mrev, mv in self._mapping._past:
+        for mrev, mv in mapp._past:
             if mrev == rev:
                 return mv == v
-        for mrev, mv in self._mapping._future:
+        for mrev, mv in mapp._future:
             if mrev == rev:
                 return mv == v
         return False
 
     def __iter__(self):
-        if self._mapping._past:
-            yield from self._mapping._past
-        if self._mapping._future:
-            yield from self._mapping._future
+        past = self._mapping._past
+        future = self._mapping._future
+        if past:
+            yield from past
+        if future:
+            yield from future
 
 
 class WindowDictPastFutureKeysView(KeysView):
-    """Abstract class for views on the past or future keys of a WindowDict"""
+    """View on a WindowDict's keys relative to last lookup"""
+    def __iter__(self):
+        if not self._mapping.stack:
+            return
+        yield from map(get0, reversed(self._mapping.stack))
+
     def __contains__(self, item):
-        if not self._mapping.deq:
-            return
-        deq = self._mapping.deq
-        if not deq or item < deq[0][0] or item > deq[-1][0]:
-            return False
-        for rev in map(itemgetter(0), deq):
-            if rev == item:
-                return True
-        return False
-
-
-class WindowDictPastKeysView(WindowDictPastFutureKeysView):
-    """View on a WindowDict's past keys relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from map(itemgetter(0), reversed(self._mapping.deq))
-
-
-class WindowDictFutureKeysView(WindowDictPastFutureKeysView):
-    """View on a WindowDict's future keys relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from map(itemgetter(0), self._mapping.deq)
+        return item in self._mapping and item in map(get0, self._mapping.stack)
 
 
 class WindowDictPastFutureItemsView(ItemsView):
-    """Abstract class for views on the past or future items of a WindowDict"""
+    def __iter__(self):
+        if not self._mapping.stack:
+            return
+        yield from reversed(self._mapping.stack)
+
     def __contains__(self, item):
-        deq = self._mapping.deq
-        if not deq or item[0] < deq[0][0] or item[0] > deq[-1][0]:
+        if self._out_of_range(item, self._mapping.stack):
             return False
-        rev, v = item
-        for mrev, mv in deq:
-            if mrev == rev:
-                return mv == v
+        i0, i1 = item
+        if i0 not in self._mapping:
+            return False
+        for j0, j1 in self._mapping.stack:
+            if i0 == j0:
+                return i1 == j1
         return False
 
 
 class WindowDictPastItemsView(WindowDictPastFutureItemsView):
-    """View on a WindowDict's past items relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from reversed(self._mapping.deq)
+    @staticmethod
+    def _out_of_range(item, stack):
+        return item[0] < stack[0][0] or item[0] > stack[-1][0]
 
 
 class WindowDictFutureItemsView(WindowDictPastFutureItemsView):
     """View on a WindowDict's future items relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from self._mapping.deq
+    @staticmethod
+    def _out_of_range(item, stack):
+        return item[0] < stack[-1][0] or item[0] > stack[0][0]
 
 
 class WindowDictPastFutureValuesView(ValuesView):
     """Abstract class for views on the past or future values of a WindowDict"""
+    def __iter__(self):
+        stack = self._mapping.stack
+        if not stack:
+            return
+        yield from map(get1, reversed(stack))
+
     def __contains__(self, item):
-        if not self._mapping.deq:
+        stack = self._mapping.stack
+        if not stack:
             return False
-        for v in map(itemgetter(1), self._mapping.deq):
-            if v == item:
-                return True
-        return False
-
-
-class WindowDictPastValuesView(WindowDictPastFutureValuesView):
-    """View on a WindowDict's past values relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from map(itemgetter(1), reversed(self._mapping.deq))
-
-
-class WindowDictFutureValuesView(WindowDictPastFutureValuesView):
-    """View on a WindowDict's future values relative to last lookup"""
-    def __iter__(self):
-        if not self._mapping.deq:
-            return
-        yield from map(itemgetter(1), self._mapping.deq)
+        return item in map(get1, stack)
 
 
 class WindowDictValuesView(ValuesView):
     """Look through all the values that a WindowDict contains."""
     def __contains__(self, value):
-        if self._mapping._past:
-            for rev, v in self._mapping._past:
+        past = self._mapping._past
+        future = self._mapping._future
+        if past:
+            for rev, v in past:
                 if v == value:
                     return True
-        if self._mapping._future:
-            for rev, v in self._mapping._future:
+        if future:
+            for rev, v in future:
                 if v == value:
                     return True
         return False
 
     def __iter__(self):
-        if self._mapping._past:
-            for rev, v in self._mapping._past:
-                yield v
-        if self._mapping._future:
-            for rev, v in self._mapping._future:
-                yield v
+        past = self._mapping._past
+        future = self._mapping._future
+        if past:
+            yield from map(get1, past)
+        if future:
+            yield from map(get1, future)
 
 
 class WindowDictPastFutureView(Mapping):
     """Abstract class for historical views on WindowDict"""
-    __slots__ = ('deq',)
+    __slots__ = ('stack',)
 
-    def __init__(self, deq):
-        self.deq = deq
+    def __init__(self, stack):
+        self.stack = stack
 
     def __len__(self):
-        if not self.deq:
+        stack = self.stack
+        if not stack:
             return 0
-        return len(self.deq)
-
-    def __getitem__(self, key):
-        if not self.deq or key < self.deq[0][0] or key > self.deq[-1][0]:
-            raise KeyError
-        for rev, value in self.deq:
-            if rev == key:
-                return value
-        raise KeyError
+        return len(stack)
 
 
 class WindowDictPastView(WindowDictPastFutureView):
     """Read-only mapping of just the past of a WindowDict"""
     def __iter__(self):
-        if not self.deq:
+        stack = self.stack
+        if not stack:
             return
-        yield from map(itemgetter(0), reversed(self.deq))
-
-    def keys(self):
-        return WindowDictPastKeysView(self)
-
-    def items(self):
-        return WindowDictPastItemsView(self)
-
-    def values(self):
-        return WindowDictPastValuesView(self)
-
-
-class WindowDictFutureView(WindowDictPastFutureView):
-    """Read-only mapping of just the future of a WindowDict"""
-    def __iter__(self):
-        if not self.deq:
-            return
-        yield from map(itemgetter(0), self.deq)
+        yield from map(get0, reversed(stack))
 
     def __getitem__(self, key):
-        if not self.deq or key < self.deq[0][0] or key > self.deq[-1][0]:
+        stack = self.stack
+        if not stack or key < stack[0][0] or key > stack[-1][0]:
             raise KeyError
-        for rev, value in self.deq:
+        for rev, value in stack:
             if rev == key:
                 return value
         raise KeyError
 
     def keys(self):
-        return WindowDictFutureKeysView(self)
+        return WindowDictPastFutureKeysView(self)
+
+    def items(self):
+        return WindowDictPastItemsView(self)
+
+    def values(self):
+        return WindowDictPastFutureValuesView(self)
+
+
+class WindowDictFutureView(WindowDictPastFutureView):
+    """Read-only mapping of just the future of a WindowDict"""
+    def __iter__(self):
+        stack = self.stack
+        if not stack:
+            return
+        yield from map(get0, reversed(stack))
+
+    def __getitem__(self, key):
+        stack = self.stack
+        if not stack or key < stack[-1][0] or key > stack[0][0]:
+            raise KeyError
+        for rev, value in stack:
+            if rev == key:
+                return value
+        raise KeyError
+
+    def keys(self):
+        return WindowDictPastFutureKeysView(self)
 
     def items(self):
         return WindowDictFutureItemsView(self)
 
     def values(self):
-        return WindowDictFutureValuesView(self)
+        return WindowDictPastFutureValuesView(self)
 
 
 class WindowDictSlice:
@@ -307,45 +295,57 @@ class WindowDictSlice:
             return
         slic = self.slice
         if slic.step is not None:
-            for i in range(slic.start or dic.beginning, slic.stop or dic.end, slic.step):
+            for i in range(slic.start or dic.beginning, slic.stop or dic.end+1, slic.step):
                 yield dic[i]
+            return
         if slic.start is None and slic.stop is None:
-            if dic._past:
-                yield from map(itemgetter(1), dic._past)
-            if dic._future:
-                yield from map(itemgetter(1), dic._future)
+            yield from map(get1, dic._past)
+            yield from map(get1, reversed(dic._future))
         elif None not in (slic.start, slic.stop):
             if slic.stop == slic.start:
                 yield dic[slic.stop]
                 return
-            left, right = (slic.start, slic.stop) if slic.start < slic.stop else (slic.stop, slic.start)
-            dic.seek(right)
-            if not dic._past:
-                return
-            past = dic._past.copy()
-            popper = getattr(past, 'popleft', lambda: past.pop(0))
-            while past and past[0][0] < left:
-                popper()
-            yield from map(itemgetter(1), past)
+            past = dic._past
+            future = dic._future
+            if slic.start < slic.stop:
+                left, right = slic.start, slic.stop
+                dic.seek(right)
+                if not past:
+                    return
+                if past[-1][0] == right:
+                    future.append(past.pop())
+                cmp = lt
+            else:
+                left, right = slic.stop, slic.start
+                dic.seek(right)
+                if not past:
+                    return
+                cmp = le
+            it = iter(past)
+            p0, p1 = next(it)
+            while cmp(p0, left):
+                p0, p1 = next(it)
+            else:
+                yield p1
+            yield from map(get1, it)
         elif slic.start is None:
-            stac = []
-            if dic._past:
-                stac.extend(dic._past)
-            if dic._future:
-                stac.extend(dic._future)
-            while stac and stac[-1][0] > slic.stop:
+            stac = dic._past + list(reversed(dic._future))
+            while stac and stac[-1][0] >= slic.stop:
                 stac.pop()
-            yield from map(itemgetter(1), stac)
+            yield from map(get1, stac)
             return
         else:  # slic.stop is None
-            stac = deque()
-            if dic._past:
-                stac.extend(dic._past)
-            if dic._future:
-                stac.extend(dic._future)
-            while stac and stac[0][0] < slic.start:
-                stac.popleft()
-            yield from map(itemgetter(1), stac)
+            if not dic._past and not dic._future:
+                return
+            chan = chain(dic._past, reversed(dic._future))
+            nxt = next(chan)
+            while nxt[0] < slic.start:
+                try:
+                    nxt = next(chan)
+                except StopIteration:
+                    return
+            yield get1(nxt)
+            yield from map(get1, chan)
 
 
 class WindowDictReverseSlice:
@@ -367,39 +367,40 @@ class WindowDictReverseSlice:
         if slic.step is not None:
             for i in range(slic.start or dic.end, slic.stop or dic.beginning, slic.step):
                 yield dic[i]
+            return
         if slic.start is None and slic.stop is None:
-            if dic._future:
-                yield from map(itemgetter(1), reversed(dic._future))
-            if dic._past:
-                yield from map(itemgetter(1), reversed(dic._past))
+            yield from map(get1, dic._future)
+            yield from map(get1, reversed(dic._past))
         elif None not in (slic.start, slic.stop):
             if slic.start == slic.stop:
                 yield dic[slic.stop]
                 return
-            left, right = (slic.start, slic.stop) if slic.start < slic.stop else (slic.stop, slic.start)
-            dic.seek(right)
-            for frev, fv in reversed(dic._past):
-                if frev <= left:
+            if slic.start < slic.stop:
+                left, right = slic.start, slic.stop
+                dic.seek(right)
+                it = reversed(dic._past)
+                next(it)
+                cmp = lt
+            else:
+                left, right = slic.stop, slic.start
+                dic.seek(right)
+                it = reversed(dic._past)
+                cmp = le
+            for frev, fv in it:
+                if cmp(frev, left):
                     return
                 yield fv
         elif slic.start is None:
-            stac = deque()
-            if dic._past:
-                stac.extend(dic._past)
-            if dic._future:
-                stac.extend(dic._future)
-            while stac and stac[-1][0] > slic.stop:
+            stac = dic._past + list(reversed(dic._future))
+            while stac and stac[-1][0] >= slic.stop:
                 stac.pop()
-            yield from map(itemgetter(1), reversed(stac))
+            yield from map(get1, reversed(stac))
         else:  # slic.stop is None
-            stac = deque()
-            if dic._past:
-                stac.extend(dic._past)
-            if dic._future:
-                stac.extend(dic._future)
+            stac = deque(dic._past)
+            stac.extend(reversed(dic._future))
             while stac and stac[0][0] < slic.start:
                 stac.popleft()
-            yield from map(itemgetter(1), reversed(stac))
+            yield from map(get1, reversed(stac))
 
 
 class WindowDict(MutableMapping):
@@ -407,10 +408,7 @@ class WindowDict(MutableMapping):
 
     Look up a revision number in this dict and it will give you the
     effective value as of that revision. Keys should always be
-    revision numbers. Once a key is set, all greater keys are
-    considered to be in this dict unless the value is ``None``. Keys
-    after that one aren't "set" until one's value is non-``None``
-    again.
+    revision numbers.
 
     Optimized for the cases where you look up the same revision
     repeatedly, or its neighbors.
@@ -420,91 +418,96 @@ class WindowDict(MutableMapping):
     values, with no indication of when they're from exactly --
     so explicitly supply a step of 1 to get the value at each point in
     the slice, or use the ``future`` and ``past`` methods to get read-only
-    mappings of data relative to when you last got an item from this.
+    mappings of data relative to a particular revision.
 
     Unlike slices of eg. lists, you can slice with a start greater than the stop
-    even if you don't supply a step. That will get you values in reverse order,
-    still without retaining the revision they're from.
+    even if you don't supply a step. That will get you values in reverse order.
 
     """
-    __slots__ = ('_future', '_past')
-
-    DEQUE_THRESHOLD = 50
-    """How long my past or future has to get before I'll turn it into a deque"""
+    __slots__ = ('_future', '_past', '_keys', '_beginning')
 
     def future(self, rev=None):
-        """Return a Mapping of items after the given revision."""
+        """Return a Mapping of items after the given revision.
+
+        Default revision is the last one looked up.
+
+        """
         if rev is not None:
             self.seek(rev)
         return WindowDictFutureView(self._future)
 
     def past(self, rev=None):
-        """Return a Mapping of items at or before the given revision."""
+        """Return a Mapping of items at or before the given revision.
+
+        Default revision is the last one looked up.
+
+        """
         if rev is not None:
             self.seek(rev)
         return WindowDictPastView(self._past)
 
+    @cython.locals(rev=cython.int, past_end=cython.int, future_start=cython.int)
     def seek(self, rev):
         """Arrange the caches to help look up the given revision."""
         # TODO: binary search? Perhaps only when one or the other
-        # deque is very large?
+        # stack is very large?
         if not self:
             return
-        if self._past and self._past[-1][0] <= rev and (
-                not self._future or self._future[0][0] > rev
-        ):
-            return
-        if self._past is None:
-            self._past = []
-        if self._future:
-            popper = getattr(self._future, 'popleft', lambda: self._future.pop(0))
-            while self._future and self._future[0][0] <= rev:
-                self._past.append(popper())
-        if self._past and self._future is None:
-            self._future = []
-        if self._past:
-            prepender = getattr(self._future, 'appendleft', lambda x: self._future.insert(0, x))
-            while self._past and self._past[-1][0] > rev:
-                prepender(self._past.pop())
+        if type(rev) is not int:
+            raise TypeError("rev must be int")
+        past = self._past
+        future = self._future
+        if future:
+            appender = past.append
+            popper = future.pop
+            future_start = future[-1][0]
+            while future_start <= rev:
+                appender(popper())
+                if future:
+                    future_start = future[-1][0]
+                else:
+                    break
+        if past:
+            popper = past.pop
+            appender = future.append
+            past_end = past[-1][0]
+            while past_end > rev:
+                appender(popper())
+                if past:
+                    past_end = past[-1][0]
+                else:
+                    break
 
-    def rev_gettable(self, rev):
-        if self._past:
-            return rev >= self._past[0][0]
-        elif self._future:
-            return rev >= self._future[0][0]
-        else:
+    def rev_gettable(self, rev: int) -> bool:
+        beg = self._beginning
+        if beg is None:
             return False
+        return rev >= beg
 
-    def rev_before(self, rev):
+    def rev_before(self, rev: int) -> int:
         """Return the latest past rev on which the value changed."""
         self.seek(rev)
         if self._past:
             return self._past[-1][0]
 
-    def rev_after(self, rev):
+    def rev_after(self, rev: int) -> int:
         """Return the earliest future rev on which the value will change."""
         self.seek(rev)
         if self._future:
-            return self._future[0][0]
+            return self._future[-1][0]
 
-    def truncate(self, rev):
+    def truncate(self, rev: int) -> None:
         """Delete everything after the given revision."""
         self.seek(rev)
-        self._future = None
+        self._keys.difference_update(map(get0, self._future))
+        self._future = []
+        if not self._past:
+            self._beginning = None
 
     @property
-    def beginning(self):
-        if self._past:
-            return self._past[0][0]
-        elif self._future:
-            return self._future[0][0]
-        else:
-            raise HistoryError("No history yet")
-
-    @property
-    def end(self):
+    def end(self) -> int:
         if self._future:
-            return self._future[-1][0]
+            return self._future[0][0]
         elif self._past:
             return self._past[-1][0]
         else:
@@ -523,39 +526,31 @@ class WindowDict(MutableMapping):
         return bool(self._past) or bool(self._future)
 
     def __init__(self, data=None):
-        if hasattr(data, 'items'):
-            if len(data) > self.DEQUE_THRESHOLD:
-                self._past = deque(sorted(data.items()))
-            else:
-                self._past = list(sorted(data.items()))
-        elif data:
-            # assume it's an orderable sequence of pairs
-            if len(data) > self.DEQUE_THRESHOLD:
-                self._past = deque(sorted(data))
-            else:
-                self._past = list(sorted(data))
+        if not data:
+            self._past = []
+        elif hasattr(data, 'items'):
+            self._past = list(data.items())
         else:
-            self._past = None
-        self._future = None
+            # assume it's an orderable sequence of pairs
+            self._past = list(data)
+        self._past.sort()
+        self._future = []
+        self._keys = set(map(get0, self._past))
+        self._beginning = None if not self._past else self._past[0][0]
 
     def __iter__(self):
         if not self:
             return
         if self._past:
-            for (rev, v) in self._past:
-                yield rev
+            yield from map(get0, self._past)
         if self._future:
-            for (rev, v) in self._future:
-                yield rev
+            yield from map(get0, self._future)
 
     def __contains__(self, item):
-        if not within_history(item, self):
-            return False
-        self.seek(item)
-        return self._past and self._past[-1][0] == item
+        return item in self._keys
 
     def __len__(self):
-        return len(self._past or ()) + len(self._future or ())
+        return len(self._past) + len(self._future)
 
     def __getitem__(self, rev):
         if not self:
@@ -565,40 +560,40 @@ class WindowDict(MutableMapping):
                 return WindowDictReverseSlice(self, rev)
             return WindowDictSlice(self, rev)
         self.seek(rev)
-        if not self._past:
+        past = self._past
+        if not past:
             raise HistoryError(
                 "Revision {} is before the start of history".format(rev)
             )
-        ret = self._past[-1][1]
-        if ret is None:
-            raise HistoryError("Set, then deleted", deleted=True)
-        return ret
+        return past[-1][1]
 
+    @cython.locals(past_end=cython.int, rev=cython.int)
     def __setitem__(self, rev, v):
         if hasattr(v, 'unwrap') and not hasattr(v, 'no_unwrap'):
             v = v.unwrap()
-        if self._past is None:
-            self._past = []
-        if not self._past and not self._future:
-            self._past.append((rev, v))
-        elif self._past and rev < self._past[0][0]:
-            self._past.insert(0, (rev, v))
-        elif self._past and rev == self._past[0][0]:
-            self._past[0] = (rev, v)
-        else:
+        past = self._past
+        if past or self._future:
             self.seek(rev)
-            if not self._past:
-                self._past.append((rev, v))
-            elif self._past[-1][0] == rev:
-                self._past[-1] = (rev, v)
+            past_end = -1 if not past else past[-1][0]
+            if not past:
+                past.append((rev, v))
+                self._beginning = rev
+            elif past_end == rev:
+                past[-1] = (rev, v)
             else:
-                assert self._past[-1][0] < rev
-                self._past.append((rev, v))
-        if type(self._past) is list and len(self._past) > self.DEQUE_THRESHOLD:
-            self._past = deque(self._past)
-        if type(self._future) is list and len(self._future) > self.DEQUE_THRESHOLD:
-            self._future = deque(self._future)
+                past.append((rev, v))
+        else:
+            past.append((rev, v))
+            self._beginning = rev
+        self._keys.add(rev)
 
+    @property
+    def beginning(self):
+        if self._beginning is None:
+            raise HistoryError("No history yet")
+        return self._beginning
+
+    @cython.locals(rev=cython.int, past_end=cython.int)
     def __delitem__(self, rev):
         # Not checking for rev's presence at the beginning because
         # to do so would likely require iterating thru history,
@@ -609,11 +604,68 @@ class WindowDict(MutableMapping):
         if not self.beginning <= rev <= self.end:
             raise HistoryError("Rev outside of history: {}".format(rev))
         self.seek(rev)
-        if self._past[-1][0] != rev:
+        past = self._past
+        past_end = -1 if not past else past[-1][0]
+        if not past or past_end != rev:
             raise HistoryError("Rev not present: {}".format(rev))
-        del self._past[-1]
+        del past[-1]
+        if not past:
+            if self._future:
+                self._beginning = self._future[-1][0]
+            else:
+                self._beginning = None
+        self._keys.remove(rev)
 
     def __repr__(self):
         me = dict(self._past)
         me.update(self._future)
         return "{}({})".format(self.__class__.__name__, me)
+
+
+class FuturistWindowDict(WindowDict):
+    """A WindowDict that does not let you rewrite the past."""
+    __slots__ = ('_future', '_past', '_beginning')
+
+    def __setitem__(self, rev, v):
+        if hasattr(v, 'unwrap') and not hasattr(v, 'no_unwrap'):
+            v = v.unwrap()
+        self.seek(rev)
+        past = self._past
+        future = self._future
+        if future:
+            raise HistoryError(
+                "Already have some history after {}".format(rev)
+            )
+        if not past:
+            self._beginning = rev
+            past.append((rev, v))
+        elif rev > past[-1][0]:
+            past.append((rev, v))
+        elif rev == past[-1][0]:
+            past[-1] = (rev, v)
+        else:
+            raise HistoryError(
+                "Already have some history after {} "
+                "(and my seek function is broken?)".format(rev)
+            )
+        self._keys.add(rev)
+
+
+class TurnDict(FuturistWindowDict):
+    __slots__ = ('_future', '_past')
+    cls = FuturistWindowDict
+
+    def __setitem__(self, turn, value):
+        if type(value) is not FuturistWindowDict:
+            value = FuturistWindowDict(value)
+        FuturistWindowDict.__setitem__(self, turn, value)
+
+
+class SettingsTurnDict(WindowDict):
+    __slots__ = ('_future', '_past')
+    cls = WindowDict
+
+    def __setitem__(self, turn, value):
+        if type(value) is not WindowDict:
+            value = WindowDict(value)
+        WindowDict.__setitem__(self, turn, value)
